<?php
/**
 * @package   	JCE
 * @copyright 	Copyright © 2009-2011 Ryan Demmer. All rights reserved.
 * @license   	GNU/GPL 2 or later - http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 * JCE is free software. This version may have been modified pursuant
 * to the GNU General Public License, and as distributed it includes or
 * is derivative of works licensed under the GNU General Public License or
 * other free or open source software licenses.
 */

defined('_JEXEC') or die('Restricted access');
defined('_WF_EXT') or die('RESTRICTED');
jimport('joomla.filesystem.file');

class VMlinks extends JObject
{
    
    var $_option = 'com_virtuemart';
	var $isVM1 = true;
    /**
     * Constructor activating the default information of the class
     *
     * @access	protected
     */
    function __construct($options = array())
    {
		if (JFile::exists(JPATH_ADMINISTRATOR.DS.'components'.DS.'com_virtuemart'.DS.'version.php' )) { 
			include_once (JPATH_ADMINISTRATOR.DS.'components'.DS.'com_virtuemart'.DS.'version.php' );
			$VMVERSION = new vmVersion();
			if (isset($VMVERSION->RELEASE)) {
				$version = $VMVERSION->RELEASE;
			} else {
				$version = vmVersion::$RELEASE;
			}

			if (version_compare($version,'2.0.0','<')) {
				$this->isVM1 = true;
				require_once(JPATH_SITE.DS.'components'.DS.'com_virtuemart'.DS.'virtuemart_parser.php' );
			} else {
				$this->isVM1 = false;
				if (!class_exists( 'VmConfig' )) require(JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_virtuemart'.DS.'helpers'.DS.'config.php');
				VmConfig::loadConfig();
			}
		} else {
			//Virtuemart not found
			return false;
		}

    }
    
    /**
     * Returns a reference to a editor object
     *
     * This method must be invoked as:
     * 		<pre>  $browser =JContentEditor::getInstance();</pre>
     *
     * @access	public
     * @return	JCE  The editor object.
     * @since	1.5
     */
    function & getInstance()
    {
        static $instance;
        
        if (!is_object($instance)) {
            $instance = new VMlinks();
        }
        return $instance;
    }
    public function getOption()
    {
        return $this->_option;
    }
    public function getList()
    {
        //Reference to JConentEditor (JCE) instance
        $wf = WFEditorPlugin::getInstance();
        
        if ($wf->checkAccess('links.vmlinks.categories', 1)) {
            return '<li id="index.php?option=com_virtuemart"><div class="tree-row"><div class="tree-image"></div><span class="folder virtuemart nolink"><a href="javascript:;">Virtuemart</a></span></div></li>';
        }
    }
	
	function getLinks($args)
	{
		if ($this->isVM1) {
			return $this->getLinksVM1($args);
		} else {
			return $this->getLinksVM2($args);
		}
	}
	
	function getLinksVM1($args)
	{
		$items 	= array();
		$parent		= isset($args->category_id) ? $args->category_id : 0;
		$page		= isset($args->page) ? $args->page : '';
		
		if ($parent==0 && $page=='') {
			$items[] = array(
				'id'		=>	'index.php?option=com_virtuemart&page=listpages',
				'name'		=>	'Pages',
				'class'		=>	'folder pages nolink'
			);
		}
		if ($parent==0 && $page=='listpages') {
			$items[] = array(
				'id'		=>	'index.php?option=com_virtuemart&view=shop.index',
				'name'		=>	'Home',
				'class'		=>	'vmhome'
			);
			$items[] = array(
				'id'		=>	'index.php?option=com_virtuemart&page=account.index',
				'name'		=>	'Account',
				'class'		=>	'vmaccount'
			);
			$items[] = array(
				'id'		=>	'index.php?option=com_virtuemart&page=account.orders',
				'name'		=>	'Orders',
				'class'		=>	'vmorders'
			);
			$items[] = array(
				'id'		=>	'index.php?option=com_virtuemart&page=shop.cart',
				'name'		=>	'Cart',
				'class'		=>	'vmcart'
			);
			$items[] = array(
				'id'		=>	'index.php?option=com_virtuemart&page=shop.registration',
				'name'		=>	'Register',
				'class'		=>	'vmregister'
			);
			$items[] = array(
				'id'		=>	'index.php?option=com_virtuemart&page=shop.search',
				'name'		=>	'Search',
				'class'		=>	'vmsearch'
			);
		}
		
		$categories = self::_getCategories($parent);
		foreach ($categories as $category) {
			$items[] = array(
				'id'		=>	'index.php?option=com_virtuemart&page=shop.browse&category_id=' . $category->category_id,
				'name'		=>	$category->category_name,
				'class'		=>	'folder vmcategory'
			);
		}

		$products = self::_getProducts($parent);
		foreach ($products as $product) {
			$items[] = array(
				'id'		=>	'index.php?option=com_virtuemart&page=shop.product_details&flypage='.$product->category_flypage.'&category_id=' . $parent.'&product_id='.$product->product_id,
				'name'		=>	$product->product_name.' / '.$product->product_sku,
				'class'		=>	'product'
			);
		}
	
		return $items;
	}

	function getLinksVM2($args)
	{
		$items 	= array();
		$parent		= isset($args->virtuemart_category_id) ? $args->virtuemart_category_id : 0;
		$view		= isset($args->view) ? $args->view : '';
		
		if ($parent==0 && $view=='') {
			$items[] = array(
				'id'		=>	'index.php?option=com_virtuemart&view=listpages',
				'name'		=>	'Pages',
				'class'		=>	'folder pages nolink'
			);
		}
		if ($parent==0 && $view=='listpages') {
			$items[] = array(
				'id'		=>	'index.php?option=com_virtuemart&view=virtuemart',
				'name'		=>	'Home',
				'class'		=>	'vmhome'
			);
			$items[] = array(
				'id'		=>	'index.php?option=com_virtuemart&view=user',
				'name'		=>	'Account',
				'class'		=>	'vmaccount'
			);
			$items[] = array(
				'id'		=>	'index.php?option=com_virtuemart&view=orders',
				'name'		=>	'Orders',
				'class'		=>	'vmorders'
			);
			$items[] = array(
				'id'		=>	'index.php?option=com_virtuemart&view=cart',
				'name'		=>	'Cart',
				'class'		=>	'vmcart'
			);
		}
		
		$categories = self::_getCategories($parent);
		foreach ($categories as $category) {
			$items[] = array(
				'id'		=>	'index.php?option=com_virtuemart&view=category&virtuemart_category_id=' . $category->virtuemart_category_id,
				'name'		=>	$category->category_name,
				'class'		=>	'folder vmcategory'
			);
		}

		$products = self::_getProducts($parent);
		foreach ($products as $product) {
			$items[] = array(
				'id'		=>	'index.php?option=com_virtuemart&view=productdetails&virtuemart_category_id='.$parent.'&virtuemart_product_id='.$product->virtuemart_product_id,
				'name'		=>	$product->product_name.' / '.$product->product_sku,
				'class'		=>	'product'
			);
		}
	
		return $items;
	}

	private function _getCategories($parent=0) {
		if ($this->isVM1) {
			$db = new ps_DB;
			$q = "SELECT c.category_id, c.category_name 
			FROM #__{vm}_category c
			INNER JOIN #__{vm}_category_xref x ON c.category_id = x.category_child_id
			WHERE c.category_publish = 'Y'
			AND x.category_parent_id = '".$parent."'
			ORDER BY c.list_order, c.category_name";
			$db->setQuery($q);   
			return $db->loadObjectList();
		} else {
			$db		= JFactory::getDBO();
			$q = "SELECT c.virtuemart_category_id, cl.category_name 
			FROM #__virtuemart_categories c
			INNER JOIN #__virtuemart_categories_".VMLANG." cl ON c.virtuemart_category_id = cl.virtuemart_category_id
			INNER JOIN #__virtuemart_category_categories x ON c.virtuemart_category_id = x.category_child_id
			WHERE c.published = 1
			AND x.category_parent_id = '".$parent."'
			ORDER BY x.ordering, c.ordering, cl.category_name";
			$db->setQuery($q);   
			return $db->loadObjectList();
		}
	}
	
	private function _getProducts($category_id) {
		if ($this->isVM1) {
			$db = new ps_DB;
			$q = "SELECT p.product_id, p.product_name, p.product_sku, c.category_flypage
			FROM #__{vm}_product p
			INNER JOIN #__{vm}_product_category_xref x ON p.product_id	 = x.product_id	
			INNER JOIN #__{vm}_category c ON c.category_id = x.category_id
			WHERE p.product_publish = 'Y'
			AND x.category_id = '".$category_id."'
			ORDER BY x.product_list, p.product_name";
			$db->setQuery($q);   
			return $db->loadObjectList();
		} else {
			$db		= JFactory::getDBO();
			$q = "SELECT p.virtuemart_product_id, pl.product_name, p.product_sku
			FROM #__virtuemart_products AS p
			INNER JOIN #__virtuemart_products_".VMLANG." pl ON p.virtuemart_product_id = pl.virtuemart_product_id
			INNER JOIN #__virtuemart_product_categories x ON p.virtuemart_product_id = x.virtuemart_product_id
			WHERE p.published = 1
			AND p.product_parent_id = 0
			AND x.virtuemart_category_id = '".$category_id."'
			ORDER BY x.ordering, pl.product_name";
			$db->setQuery($q);   
			return $db->loadObjectList();
		}
	}
	
}
?>